#  Copyright (c) 2022. Lorem
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
#  associated documentation files (the "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
#  of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
#  conditions:
#
#  The above copyright notice and this permission notice shall be included in all copies or substantial
#  portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
#  PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
#  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
#  OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#  OTHER DEALINGS IN THE SOFTWARE.
from http.client import HTTPSConnection


class ApiConnection(HTTPSConnection):
    def __init__(self, host):
        """
        This is the Base connection class for generel wrappers that we maybe want to write.
        """

        super().__init__(host=host)

    def get(self, url: str, headers: dict) -> tuple:
        """ This will send a `GET` request

        Parameters
        ----------
        url : :class:`str`
            The url to send the request to.

        headers : :class:`dict`
            The headers to send with the request. This is for the jwt token
            and to tell the API what we want back from them.

        Returns
        -------
        :class:`bytes`
            The read response from the server.
        Notes
        -----
        The JWT token will be only used for non-read-only requests.
        """

        res, header = self.__request('GET', headers, url)

        self.close()
        return res, header

    def __request(self, method: str, headers: dict, url: str) -> tuple:
        """ This is just for preventing repetitive code samples

        Parameters
        ----------
        method : str
            The request method that is needed, e.x. POST or GET.

        headers : dict
            The default header for the authorization.

        url : str
            The url to send the request to.

        Returns
        -------
        bytes and HTTPMessage
            Returns the data
        """

        self.request(method, url, headers=headers)

        data = self.getresponse()
        res = data.read()
        header = data.headers

        return res, header

    def __requests_body(self, method: str, data: dict, headers: dict, url: str) -> tuple:
        """ the same as the __request method but for PUT and POST req

        Parameters
        ----------
        method : :class:`str`
            The method that you want to use, possible values POST or PUT.

        data : :class:`dict`
            The body that will be delivered with the request.

        headers : :class:`dict`
            The default headers for authorization.

        url : :class:`str`
            The endpoints url to send the request to.

        Returns
        -------
        :class:`bytes` and :class:`HTTPMessage`
            The response that the api gives us back.
        """

        self.request(method, url, headers=headers, body=data)

        rdata = self.getresponse()
        res = rdata.read()
        header = rdata.headers

        return res, header

    def post(self, url: str, headers: dict, data: dict) -> tuple:
        """
        This will send a `POST` request

        Parameters
        ----------
        url : str
            The url to send the request to.

        headers : dict
            The headers to send with the request. This is for the jwt token
            and accept thingies.

        data : dict
            The body of the request. This is the data that will be sent to
            the server.

        Returns
        -------
        bytes and HTTPMessage
            The read response from the server.
        """

        res, header = self.__requests_body('POST', data=data, headers=headers, url=url)

        self.close()
        return res, header

    def delete(self, url: str, headers: dict) -> tuple:
        """
        This will send a `DELETE` request

        Parameters
        ----------
        url : str
            The url to send the request to.

        headers : dict
            The headers to send with the request. This is for the jwt token
            and accept thingies.

        Returns
        -------
        bytes and HTTPMessage
            The read response from the server. When it responds something.
        """

        res, header = self.__request('DELETE', url=url, headers=headers)

        self.close()
        return res, header

    def put(self, url: str, headers: dict, data: dict) -> tuple:
        """ Here you can create entries on the api

        Parameters
        ----------
        url : str
            The endpoints url to access.

        headers : dict
            The default headers.

        data : dict
            The data for the creation.

        Returns
        -------
        bytes and HTTPMessage
            The response from the endpoint.
        """
        res, header = self.__requests_body('PUT', data=data, url=url, headers=headers)

        self.close()
        return res, header



import enum
from datetime import datetime
from typing import TYPE_CHECKING

import pytz

if TYPE_CHECKING:
    from .logger import Logger


class LogLevel(enum.IntEnum):
    INFO = 1
    WARNING = 2
    ERROR = 3
    DEBUG = 4


class Ansi(enum.Enum):
    RED = '\u001b[31m'
    YELLOW = '\u001b[33m'
    GREEN = '\u001b[32m'
    RESET = '\u001b[0m'


def write_logfile(logger: 'Logger', msg) -> int:
    if logger.log_file is None:
        return 0

    with open(logger.log_file, 'a+', encoding='utf-8') as f:
        f.write(msg + '\n')

    return 0


def create_prefix(date: bool,
                  name: str,
                  _type: str = '',
                  tz: str = 'Europe/Berlin') -> str:
    ret = ''
    if date:
        ret += f'{datetime.now(tz=pytz.timezone(tz)):%d-%m-%Y/%H:%M:%S} | '

    if _type:
        ret += f'{_type} | '

    ret += f'{name} | '
    return ret



#  Copyright (c) 2022. exersalza
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
#  associated documentation files (the "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
#  of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following
#  conditions:
#
#  The above copyright notice and this permission notice shall be included in all copies or substantial
#  portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
#  PURPOSE AND NON INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
#  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
#  OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#  OTHER DEALINGS IN THE SOFTWARE.
from __future__ import annotations

import socket
from collections import namedtuple

from .utils import create_prefix, Ansi, LogLevel, write_logfile

level = namedtuple('level', ['level', 'color'])

log_levels = {
    'info': level(1, Ansi.GREEN),
    'warn': level(2, Ansi.YELLOW),
    'error': level(3, Ansi.RED),
    'debug': level(4, Ansi.RESET)
}


class Logger:
    def __init__(self, log_level: int | LogLevel,
                 name: str = '',
                 log_file: str = None,
                 prefix: str = '',
                 disp_type: bool = True) -> None:

        """ **This is the base Logger class.**

        Parameters:
        ----------
        log_level : int
            Set the log level.<br>
            0: None, 1: Info, 2: Warning, 3: Error, 4: Debug

        name : str, optional
            Set the name for the Logger. <br>
            Default: hostname of the machine.

        log_file : str, optional
            Set the path to a Logfile.

        prefix : str, optional
            Create a Prefix that appears in front of the msg. <br>
            You can set a completely custom prefix, but also generate one from the create_prefix function.
            <br>[default]: DD-MM-YYYY/HH:MM:SS | hostname | msg

        disp_type : bool, optional
            Should the Logger display the type of the message.
        """

        self.log_file = log_file
        self.log_level = log_level
        self.prefix = prefix
        self.disp_type = disp_type

        self.hostname = name if name else socket.gethostname()

    def __prepare_mod_msg(self, msg: str, name: str) -> str:
        """ Prepare the Log message. Appends the Prefix and color codes

        Parameters
        ----------
        msg : str
            The base message from the User.
        name : str
            The name of the function that called it.

        Returns
        -------
        str
            The modified message from the User.

        """
        prefix = self.prefix

        if not self.prefix:
            prefix = create_prefix(True, self.hostname, _type=name if self.disp_type else '')

        mod_msg = log_levels[name].color.value + prefix + msg + Ansi.RESET.value

        write_logfile(self, prefix + msg)
        return mod_msg

    def __can_log(self, name) -> bool:
        """ This function checks if the log level is says it can log.

        Parameters
        ----------
        name : str
            The name of the log function that was called.

        Returns
        -------
        bool
            Does the loglevel match the required level
        """

        if not log_levels[name][0] <= self.log_level:
            return False
        return True

    def info(self, msg: str) -> None:
        """ This represents a Green message with info content. """
        if self.__can_log('info'):
            print(self.__prepare_mod_msg(msg, 'info'))

    def warning(self, msg: str) -> None:
        """ This represents a Yellow message with warning content. """
        if self.__can_log('warn'):
            print(self.__prepare_mod_msg(msg, 'warn'))

    def error(self, msg: str) -> None:
        """ This represents a Red message with error content. """
        if self.__can_log('error'):
            print(self.__prepare_mod_msg(msg, 'error'))

    def debug(self, msg: str) -> None:
        """ This represents a White message with Debug content. """
        if self.__can_log('error'):
            print(self.__prepare_mod_msg(msg, 'debug'))



from __future__ import annotations

from typing import Any

from mysql.connector import connect


class Connection:
    def __init__(self, **kw) -> None:
        self.user = kw.get('user', 'root')
        self.password = kw.get('password', '')
        self.host = kw.get('host', 'localhost')
        self.db = kw.get('db', '')

    def get(self, sql: str, params: Any = '') -> list[dict | tuple] | None:
        """ Here we can select something from the Database.

        Parameters
        ----------
        sql : :class:`str`
            An sql string to select something from the Database.

        params : :class:`Any`
            The parameters for the sql execution.

        Returns
        -------
        list | List[Optional[dict]] | List[Tuple]
            Returns the result of the given query.
        """

        conn = connect(user=self.user, password=self.password, host=self.host, database=self.db)
        cur = conn.cursor(buffered=True)

        cur.execute(sql, params)
        result = cur.fetchall()

        conn.commit()
        cur.close()
        return result

    def update(self, sql: str, params: Any = '') -> Any:
        """ With this method you can Update the Database, like `UPDATE` or `DELETE` query's

        Parameters
        ----------
        sql : :class:`str`
            An sql string to select something from the Database.

        params : :class:`Any`
            The parameters for the sql execution.

        Returns
        -------
        None
            Nothing.
        """

        return self.get(sql, params)

    async def aget(self, sql: str, params: Any = '') -> list[dict | tuple | None]:
        """ Here we can select asynchronous something from the Database.

        Parameters
        ----------
        sql : :class:`str`
            An sql string to select something from the Database.

        params : :class:`Any`
            The parameters for the sql execution.

        Returns
        -------
        list | List[Optional[dict]] | List[Tuple]
            Returns the result of the given query.
        """

        return self.get(sql, params)

    async def aupdate(self, sql: str, params: Any = '') -> Any:
        """ With this method you can Update the Database, such things like `UPDATE` or `DELETE` query's

       Parameters
       ----------
       sql : :class:`str`
           An sql string to select something from the Database.

       params : :class:`Any`
           The parameters for the sql execution.

       Returns
       -------
       None
           Nothing.
       """

        return self.update(sql, params)
